import sys, os, glob, yaml, shutil
import click

import assnake.cli.commands.dataset_commands as dataset_commands # snakemake makes it slow
import assnake.cli.commands.init as commands_init
from assnake.cli.commands.cmd_run import run

from assnake.utils import read_yaml, check_if_assnake_is_initialized, get_config_loc
from assnake.cli.cli_utils import sample_set_construction_options, add_options, generic_command_individual_samples, generate_result_list
from pkg_resources import iter_entry_points 


#---------------------------------------------------------------------------------------
#                                    CLI  initialization
#---------------------------------------------------------------------------------------

@click.group()
@click.version_option()
@click.pass_context
def cli(ctx):
    """\b
   ___    ____   ____   _  __   ___    __ __   ____
  / _ |  / __/  / __/  / |/ /  / _ |  / //_/  / __/
 / __ | _\ \   _\ \   /    /  / __ | / ,<    / _/  
/_/ |_|/___/  /___/  /_/|_/  /_/ |_|/_/|_|  /___/


\b  
O---o Welcome to the Assnake, Traveller!
 O-o  If you found yorself here, 
  O   than you are exploring the land of the MICROBIOME.
 o-O  Here you can find weapons and spells
o---O that will help you to analyse your data.
O---o The tools that are presented here 
 O-o  are for metagenomics studies on ILLUMINA data.
  O   You can check quality and preprocess your data, 
 o-O  assemble it, map, bin the contigs, 
o---O make taxonomic annotations, functional annotations,
O---o
 O-o  and work with 16s rRNA data.
  O
 o-O
o---O

\b
Please, feel free to check out all the commands. 
Please, read the help messages and if you have any questions, 
write me directly on my email fedorov.de@gmail.com, 
or create an issue or PR on GitHub.
\b
Start by initializing ASSNAKE with
assnake init command
\b
Here is it how it goes.
Somewhere on your filesystem you create a folder, and put your reads inside the ./<your_folder>/reads/raw folder.
<your_folder> is also the name of the Dataset, so choose wisely!
Than register yor dataset in the assnake with
assnake dataset create"""


    dir_of_this_file = os.path.dirname(os.path.abspath(__file__))
    config_loc = get_config_loc() 

    if config_loc is None or not os.path.isfile(config_loc):
            pass
    else:
        config = read_yaml(config_loc)
        wc_config = read_yaml(os.path.join(dir_of_this_file, '../snake/wc_config.yaml'))
        ctx.obj = {'config': config, 'wc_config': wc_config, 'requested_dfs': [], 'requests': [], 'sample_sets': []}


#---------------------------------------------------------------------------------------
#                                  assnake  INIT ***  group
#---------------------------------------------------------------------------------------

@cli.group(name='init')
def init_group():
    """Commands to initialize the ASSNAKE\n
    \bYou need to configure where assnake will store it's data and download databases.
    Assnake has internal configuration file 
    """
    pass

init_group.add_command(commands_init.init_start)
init_group.add_command(commands_init.current_config)

#---------------------------------------------------------------------------------------
#                                  assnake  DATASET ***  group
#---------------------------------------------------------------------------------------
@cli.group(chain=True)
def dataset():
    """Commands to work with datasets"""
    check_if_assnake_is_initialized()

dataset.add_command(dataset_commands.df_list)
dataset.add_command(dataset_commands.df_info)
dataset.add_command(dataset_commands.df_init)
dataset.add_command(dataset_commands.df_create)
dataset.add_command(dataset_commands.df_import_reads)
dataset.add_command(dataset_commands.df_delete)
dataset.add_command(dataset_commands.rescan_dataset)

#---------------------------------------------------------------------------------------
#                                  assnake  RESULT ***  group
#---------------------------------------------------------------------------------------
@cli.group(chain = True, help = 'Used to request and run results')
def result():
    """Commands to analyze your data"""
    check_if_assnake_is_initialized()


for entry_point in iter_entry_points('assnake.plugins'):
    module_class = entry_point.load()
    for cmd in module_class.invocation_commands:
        result.add_command(cmd)
    for cmd in module_class.initialization_commands:
        init_group.add_command(cmd)
    for snakeresult in module_class.results:
        result.add_command(snakeresult.invocation_command)


result.add_command(run)


@click.command('sample-set', short_help='Filter and trim your reads with dada2 trimmer')
@add_options(sample_set_construction_options)
@click.option('--params', help='Parameters to use', default='def', type=click.STRING )
@click.option('--message', '-m', multiple=True)

@click.pass_obj
def request_sample_set(config, message, **kwargs):
    click.echo('\n'.join(message))
    sample_set, sample_set_name = generic_command_individual_samples(config,  **kwargs)
    config['sample_sets'].append(sample_set.samples_pd.copy())
result.add_command(request_sample_set)


def main():
    cli()




#############################################
#   Perform assnake  ***  RUN cli command   #
#############################################

import click, os
from assnake.utils import get_config_loc, load_config_file
from assnake.api.loaders import update_fs_samples_csv


#---------------------------------------------------------------------------------------
#                                     RUN
#---------------------------------------------------------------------------------------

@click.command('run', short_help='Runs snakemake for requested results')
@click.option('--threads','-t', help='Threads per job', default=4)
@click.option('--jobs','-j', help='Number of jobs', default=1)
@click.option('--drmaa/--no-drmaa', default=False)
@click.option('--run/--no-run', default=False)
@click.option('--touch/--no-touch', default=False)
@click.pass_obj

def run(config, threads, jobs, drmaa, run, touch):
    import snakemake # Moved import here because it is slow as fucking fuck
    # print(config['requests'])

    click.secho('-----===RUN SNAKEMAKE===-----', bg='green', fg='black')

    curr_dir = os.path.abspath(os.path.dirname(__file__))
    # click.echo('Current dir: ' + curr_dir)
    # click.echo('Number of jobs torun in parallel: ' + str(jobs))
    drmaa_param = None
    if drmaa:
        drmaa_param=' -V -S /bin/bash -pe make {threads}'.format(threads=threads)
    status = snakemake.snakemake(os.path.join(curr_dir, '../../snake/snake_base.py'), 
        # config = config['wc_config'],    
        targets=config['requests'], 
        printshellcmds=True,
        dryrun=not run, 
        # config = load_config_file(),
        configfiles=[get_config_loc()],
        drmaa_log_dir = config['config']['drmaa_log_dir'],
        use_conda = True,
        latency_wait = 120,
        conda_prefix = config['config']['conda_dir'],
        drmaa=drmaa_param,
        touch = touch,
        cores=jobs, nodes=jobs)
    if run:
        print(config['requested_dfs'])
        if type(config['requested_dfs']) is list:
            for requested_df in config['requested_dfs']:
                update_fs_samples_csv(requested_df)
        else:
            update_fs_samples_csv(config['requested_dfs'])


#\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\


#
#
#

import click, os
@click.command('prepare_set_for_assembly')
@click.option('--df', '-d')
@click.option('--preproc','-p', help='Preprocessing to use', required = False)
@click.option('--samples-to-add','-s', 
                help='Samples from dataset to process', 
                default='', 
                metavar='<samples_to_add>', 
                type=click.STRING )
@click.option('--set_name','-n', help='Name of your sample set' )


def prepare_set_for_assembly(df, preproc, samples_to_add,set_name):
    print(df)
    print(preproc)
    print(samples_to_add)
    if preproc is None:
        preproc = 'raw'
    samples_to_add = [] if samples_to_add == '' else [c.strip() for c in samples_to_add.split(',')]
    df = assnake.api.loaders.load_df_from_db(df)
    ss = assnake.SampleSet.SampleSet(df['fs_prefix'], df['df'], preproc, samples_to_add=samples_to_add)

    click.echo(tabulate(ss.samples_pd[['fs_name', 'reads', 'preproc', 'df']].sort_values('reads'), 
        headers='keys', tablefmt='fancy_grid'))

    set_dir = os.path.join(df['fs_prefix'], df['df'], 'assembly', set_name)
    os.makedirs(set_dir, exist_ok=True)

    set_loc = os.path.join(set_dir, 'sample_set.tsv')
    ss.samples_pd[['df', 'preproc', 'fs_name']].to_csv(set_loc, sep='\t', index=False)




import assnake.api.loaders
import assnake
from tabulate import tabulate
import click, os, datetime
import pandas as pd

# https://stackoverflow.com/a/40195800
sample_set_construction_options = []

sample_set_construction_options = [
    click.option('--df','-d', help='Name of the dataset', required=True ),
    click.option('--preproc','-p', help='Preprocessing to use' ),

    click.option('--meta-column', '-c', help='Select samples based on metadata column' ),
    click.option('--column-value','-v', help='Value of metadata column by which select samples' ),

    click.option('--samples-to-add','-s', 
                help='Samples from dataset to process', 
                default='', 
                metavar='<samples_to_add>', 
                type=click.STRING ),
    click.option('--exclude-samples','-x', 
                help='Exclude this samples from run', 
                default='', 
                metavar='<samples_to_add>', 
                type=click.STRING ),
]

options_w_params = [
    click.option('--df','-d', help='Name of the dataset', required=True ),
    click.option('--preproc','-p', help='Preprocessing to use' ),

    click.option('--meta-column', '-c', help='Select samples based on metadata column' ),
    click.option('--column-value','-v', help='Value of metadata column by which select samples' ),

    click.option('--samples-to-add','-s', 
                help='Samples from dataset to process', 
                default='', 
                metavar='<samples_to_add>', 
                type=click.STRING ),
    click.option('--exclude-samples','-x', 
                help='Exclude this samples from run', 
                default='', 
                metavar='<samples_to_add>', 
                type=click.STRING ),
    click.option('--params', 
                help='Parameters to use', 
                default='def',
                type=click.STRING )
]


def add_options(options):
    def _add_options(func):
        for option in reversed(options):
            func = option(func)
        return func
    return _add_options

def generic_command_dict_of_sample_sets(config, df, preproc, meta_column, column_value, samples_to_add, exclude_samples, **kwargs):
    df_loaded = assnake.api.loaders.load_df_from_db(df)
    
    sample_sets_dict = {}

    meta_loc = os.path.join(df_loaded['fs_prefix'], df_loaded['df'], 'mg_samples.tsv')
    if os.path.isfile(meta_loc):
        meta = pd.read_csv(meta_loc, sep = '\t')
        if meta_column is not None:
            if column_value is not None:
                sample_set, sample_set_name = generic_command_individual_samples(config,  df, preproc, meta_column, column_value, samples_to_add, exclude_samples, **kwargs)
                sample_sets_dict.update({sample_set_name: sample_set})
            else: # treat empty column_value as creating multiple sample_sets for each column_value
                column_values = list(meta[meta_column].unique())
                for column_value in column_values:
                    sample_set, sample_set_name = generic_command_individual_samples(config,  df, preproc, meta_column, column_value, samples_to_add, exclude_samples, **kwargs)
                    sample_sets_dict.update({sample_set_name: sample_set})
        else:
            sample_set, sample_set_name = generic_command_individual_samples(config,  df, preproc, meta_column, column_value, samples_to_add, exclude_samples, **kwargs)
            sample_sets_dict.update({sample_set_name: sample_set})
    else:
        sample_set, sample_set_name = generic_command_individual_samples(config,  df, preproc, meta_column, column_value, samples_to_add, exclude_samples, **kwargs)
        sample_sets_dict.update({sample_set_name: sample_set})

    return sample_sets_dict

def generic_command_individual_samples(config, df, preproc, meta_column, column_value, samples_to_add, exclude_samples, **kwargs):
    """
    Construct sample sets, has multiple options.
    Returns dict or sample_sets based on the provided options.
    
    meta_column - factor column in sample metadata sheet. Cannot be combined with samples_to_add. exclude_samples has higher proirity. 
    column_value - value of column to select by. \
        Can be multiple - separated by commas without whitespace. \
        If meta_column is provided, bot no column_value is provided \
        - treat it like select all unique values of that column. 
        If multiple - one value - one sample_set. If --merge enabled - all values go in one sample_set

    assnake result request megahit -d FMT_FHM -c source run 

    """
    exclude_samples = [] if exclude_samples == '' else [c.strip() for c in exclude_samples.split(',')]
    samples_to_add = [] if samples_to_add == '' else [c.strip() for c in samples_to_add.split(',')]

    df_loaded = assnake.api.loaders.load_df_from_db(df)
    config['requested_dfs'] += [df_loaded['df']]

    # Now for the meta column stuff
    meta_loc = os.path.join(df_loaded['fs_prefix'], df_loaded['df'], 'mg_samples.tsv')
    if os.path.isfile(meta_loc):
        meta = pd.read_csv(meta_loc, sep = '\t')
        if meta_column is not None:
            if column_value is not None:
                subset_by_col_value = meta.loc[meta[meta_column] == column_value]
                if len(subset_by_col_value) > 0:
                    samples_to_add = list(subset_by_col_value['new_sample_name'])



    sample_set = assnake.api.loaders.load_sample_set(config['wc_config'], df_loaded['fs_prefix'], df_loaded['df'], preproc, samples_to_add=samples_to_add)
    if len(exclude_samples) > 0 :  
        sample_set = sample_set.loc[~sample_set['fs_name'].isin(exclude_samples), ]

    # click.echo(tabulate(sample_set[['fs_name', 'reads', 'preproc']].sort_values('reads'), headers='keys', tablefmt='fancy_grid'))

    # construct sample set name for fs
    if meta_column is None and column_value is None:
        curr_date = datetime.datetime.now()
        def_name = '{month}{year}'.format(month=curr_date.strftime("%b"), year=curr_date.strftime("%y"))
        sample_set_name = def_name
    else:
        sample_set_name = meta_column + '__' + column_value

    return sample_set, sample_set_name
    

def generate_result_list(sample_set, wc_str, df, preproc, meta_column, column_value, samples_to_add, exclude_samples, **kwargs):
    res_list = []
    for s in sample_set.to_dict(orient='records'):
        preprocessing = s['preproc']
        res_list.append(wc_str.format(
            fs_prefix = s['fs_prefix'].rstrip('\/'),    
            df = s['df'],
            preproc = preprocessing,
            sample = s['fs_name'],
            **kwargs
        ))
    return res_list

def prepare_sample_set_tsv_and_get_results(sample_set_dir_wc, result_wc, df, sample_sets, overwrite,**kwargs):
    res_list = []

    df_loaded = assnake.api.loaders.load_df_from_db(df)

    for sample_set_name in sample_sets.keys():
        sample_set_dir = sample_set_dir_wc.format(fs_prefix = df_loaded['fs_prefix'], df = df, sample_set = sample_set_name)
        sample_set_loc = os.path.join(sample_set_dir, 'sample_set.tsv')

        print(sample_set_loc)
        sample_set = sample_sets[sample_set_name]
        if not os.path.exists(sample_set_dir):
            os.makedirs(sample_set_dir, exist_ok=True)

        if not os.path.isfile(sample_set_loc):
            sample_set.to_csv(sample_set_loc, sep='\t', index=False)
        else:
            click.secho('Sample set with this name already exists!')
            if overwrite:
                sample_set.to_csv(sample_set_loc, sep='\t', index=False)
                click.secho('Overwritten')

        res_list += [result_wc.format(
            fs_prefix = df_loaded['fs_prefix'],
            df = df_loaded['df'],
            sample_set = sample_set_name,
            **kwargs
        )]

    return res_list


import scipy.cluster.hierarchy as shc

import plotly

# plotly.offline.init_notebook_mode()
import plotly.graph_objs as go
from matplotlib import colors as mcolors
import plotly.figure_factory as FF
import pandas as pd
# from skbio.stats.composition import *
from scipy.spatial.distance import pdist, squareform
    
def plot_centr(centr, subtitle=''):
    trace4 = go.Bar(
        y=list(centr.index),
        x=list(centr['uncl']/centr['total']),
        name='uncl',
        orientation = 'h',
    )
    trace2 = go.Bar(
        y=list(centr.index),
        x=list(centr['bacteria']/centr['total']),
        name='bacteria',
        orientation = 'h',
    )
    trace3 = go.Bar(
        y=list(centr.index),
        x=list(centr['other']/centr['total']),
        name='other',
        orientation = 'h',
    )
    trace1 = go.Bar(
        y=list(centr.index),
        x=list(centr['homo']/centr['total']),
        name='HUMAN',
        orientation = 'h',
    )
    trace5 = go.Bar(
        y=list(centr.index),
        x=list(centr['vir']/centr['total']),
        name='vir',
        orientation = 'h'
    )
    trace6 = go.Bar(
        y=list(centr.index),
        x=list(centr['archaea']/centr['total']),
        name='archaea',
        orientation = 'h'
    )

    data = [trace4, trace2, trace3, trace1, trace5, trace6]
    layout = go.Layout(
        barmode='stack',
        title = 'General taxa composition; ' + subtitle 
    )

    fig = go.Figure(data=data, layout=layout)
    if False: plotly.offline.iplot(fig)

def plot_mds(mds, feature_name, meta, title='MDS', select_by='fs_name'):
    features = set(meta[feature_name])
    colors = list(mcolors.CSS4_COLORS.values())
    traces = []

    for feature in features:
        samples_for_source = list(meta.loc[meta[feature_name] == feature][select_by])
        mds_sub = mds[mds.index.isin(samples_for_source)]
        trace = go.Scatter3d(
                    x = mds_sub[0], y = mds_sub[1], z = mds_sub[2],
                    mode = 'markers',
                    marker = dict(
                        size = 6,
                    ),
                    name = feature,
                    text = mds_sub.index)
        traces.append(trace)

    layout = dict(title = title,
          yaxis = dict(zeroline = False, title= 'MDS1',),
          xaxis = dict(zeroline = False, title= 'RMDS2',)
         )

    fig = go.Figure(data=traces,
                    layout=layout)
    if False: plotly.offline.iplot(fig)



def plot_reads_count_change(read_table, preprocs, sort, title = 'Reads number', trace_names = [], number_index_hack=False, plot=False):
    """
    This function plots bar chart with reads lost on each preprocessing step.

    Args:
        read_table (:obj:`pandas.DataFrame`): DataFrame with columns with number of reads for each sample on every preprocessing step
        preprocs (list[str]): List of preprocessings in called order
        sort (list[str]): Sort by this variable
    """
    read_table.index = read_table.index.map(str) +'_'
    read_table = read_table.sort_values(sort, ascending=False)

    if len(trace_names) == 0:
        trace_names = preprocs
        
    prev_preproc = preprocs[0]
    traces = []
    for p in preprocs[1:]:
        read_table['diff_'+prev_preproc+'--'+p] = read_table[prev_preproc] - read_table[p]
        traces.append(go.Bar(x=read_table.index, y=read_table['diff_'+prev_preproc+'--'+p], name='diff_'+prev_preproc+'--'+p))
        prev_preproc = p

    
    traces.append(go.Bar( x=read_table.index, y=read_table[preprocs[-1]], name=preprocs[-1]))

    traces.reverse()
    
    layout = go.Layout( barmode='stack', margin=go.layout.Margin( b=100 ), width=1800, title = title)
    fig = go.Figure(data=traces, layout=layout)
    if plot: plotly.offline.iplot(fig)


import argparse
import os.path
import os
import sys
import glob
import fnmatch
from assnake.api.fs_helpers import create_links

"Script for importing files to the assnake. Creates {prefix}/{df}/reads/{preproc}/{sample_name}"


def find_files(base, pattern):
    """
    Return list of files matching pattern in base folder.
    """
    return [n for n in fnmatch.filter(os.listdir(base), pattern) if
            os.path.isfile(os.path.join(base, n))]


def get_sample_dict_from_dir(loc, sample, variant, ext):
    temp_samples_dict = {'sample_name': sample,
                         'files': {'R1': '', 'R2': '', 'S': []},
                         'renamed_files': {'R1': '', 'R2': '', 'S': []}}

    for strand in ['R1', 'R2']:
        st = variant['strands'][strand]
        st_file = find_files(loc, sample + st + ext)
        if len(st_file) == 1:
            stripped = st_file[0].replace(variant['strands'][strand] + ext, '')
            stripped += '_' + strand + ext
            temp_samples_dict['renamed_files'][strand] = stripped
            temp_samples_dict['files'][strand] = st_file[0]

    return temp_samples_dict


def get_samples_from_dir(loc):
    """
    Searches for samples in loc. Sample should contain R1 and R2
    :param loc: location on filesystem where we should search
    :return: Returns sample dict in loc
    """
    samples_list = []
    ext = '.fastq.gz'
    end_variants = [{'name': 'normal', 'strands': {'R1': '_R1', 'R2': '_R2'}},
                    {'name': 'ILLUMINA_long', 'strands': {'R1': '_L001_R1_001', 'R2': '_L001_R2_001'}},
                    {'name': 'ILLUMINA', 'strands': {'R1': '_R1_001', 'R2': '_R2_001'}},
                    {'name': 'SRA', 'strands': {'R1': '_1', 'R2': '_2'}}]

    # Fool check
    if loc[-1] != '/':
        loc += '/'

    for variant in end_variants:
        R1 = variant['strands']['R1']  # Get first strand. For paired end data it doesn't matter.
        samples = [
            item.split("/")[-1].split(ext)[0].replace(R1, '')
            for item in glob.glob(loc + '*' + R1 + ext)
        ]
        if len(samples) > 0:
            for sample in samples:
                samples_list.append(get_sample_dict_from_dir(loc, sample, variant, ext))
            break

    return samples_list


# synonymous function was found in api.fs_helpers
'''def create_links(dir_with_reads, original_dir, sample):
    orig_wc = '{orig_dir}/{sample_file}'
    new_dir_wc = '{reads_dir}/{sample_name}'
    new_file_wc = '{new_dir}/{sample_file}'

    new_dir = new_dir_wc.format(reads_dir = dir_with_reads, sample_name = sample['sample_name'])
    if not os.path.isdir(new_dir):
        os.makedirs(new_dir)
        
    src_r1 = orig_wc.format(orig_dir = original_dir, sample_file = sample['files']['R1'])
    dst_r1 = new_file_wc.format(new_dir=new_dir, sample_file=sample['renamed_files']['R1'])

    src_r2 = orig_wc.format(orig_dir = original_dir, sample_file = sample['files']['R2'])
    dst_r2 = new_file_wc.format(new_dir=new_dir, sample_file=sample['renamed_files']['R2'])

    os.symlink(src_r1, dst_r1)
    os.symlink(src_r2, dst_r2)
'''


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument('--original_dir', type=str, help='Directory with raw reads')
    parser.add_argument('--prefix', type=str, help='Prefix on file system for dataset')
    parser.add_argument('--df', type=str, help='Name of yor dataset')
    parser.add_argument('--preproc', type=str, default='raw', help='First preprocessing')
    parser.add_argument('--dry', default=False, action='store_true',
                        help='Only print what is going to happen, without actually doing anything')
    parser.add_argument('--samples', nargs='+', help='List of sample names to import', default=[], required=False)

    args = parser.parse_args()

    # FIND SAMPLES IN DIRECTORY
    samples = get_samples_from_dir(args.original_dir)
    print(samples)
    # CREATE DIRECTORY FOR DF
    dir_with_reads = os.path.join(args.prefix, args.df, 'reads', args.preproc)
    print(dir_with_reads)
    # check if it doesn't exist
    if args.dry:
        print('Creating', dir_with_reads)
    else:
        if not os.path.isdir(dir_with_reads):
            os.makedirs(dir_with_reads)

    orig_wc = '{orig_dir}/{sample_file}'
    new_dir_wc = '{reads_dir}/{sample_name}'
    new_file_wc = '{new_dir}/{sample_file}'

    if len(args.samples) == 0:
        args.samples = [s['sample_name'] for s in samples]
    # CREATE SYMLINKS
    for s in samples:
        if len(args.samples) > 0:
            if s['sample_name'] in args.samples:

                new_dir = new_dir_wc.format(reads_dir=dir_with_reads, sample_name=s['sample_name'])

                src_r1 = orig_wc.format(orig_dir=args.original_dir, sample_file=s['files']['R1'])
                dst_r1 = new_file_wc.format(new_dir=new_dir, sample_file=s['renamed_files']['R1'])

                src_r2 = orig_wc.format(orig_dir=args.original_dir, sample_file=s['files']['R2'])
                dst_r2 = new_file_wc.format(new_dir=new_dir, sample_file=s['renamed_files']['R2'])

                if args.dry:
                    print('########', s['sample_name'], '########')
                    print('  ', new_dir)
                    print('  ', src_r1, '--->', dst_r1)
                    print('  ', src_r2, '--->', dst_r2)
                    print(s)

                else:
                    if not os.path.isdir(new_dir):
                        os.makedirs(new_dir)
                    os.symlink(src_r1, dst_r1)
                    os.symlink(src_r2, dst_r2)


if __name__ == "__main__":
    main()



from assnake.core.sample_set import SampleSet
from assnake.core.dataset import Dataset
from assnake.core.snake_module import SnakeModule

from assnake.api.loaders import load_dfs_from_db

import assnake.api.loaders as loaders


import yaml, os, pkg_resources, glob
import pandas as pd
import assnake.api.loaders as loaders
import assnake.utils

class SampleSet:
    """
    Class that agglomerates samples and provides convinience functions for different tasks, 
    such as constructing list of desired results locations, or preparing lists of files for rules.
    
    Attributes:
        samples_pd (:obj:`pandas.DataFrame`): Pandas DataFrame with information about samples
    """

    # prefix, df, preproc, fs_name
    samples_pd = pd.DataFrame(columns=['df', 'fs_name', 'preproc', 'reads', 'sample'])
    reads_info = pd.DataFrame()

    # ==Do we really need this here?==
    wc_config = {}
    config = {}

    
    def __init__(self, fs_prefix, df, preproc, samples_to_add = [], do_not_add = [], pattern = ''):

        self.wc_config = assnake.utils.load_wc_config()
        self.config = assnake.utils.load_config_file()

        discovered_plugins = {
            entry_point.name: entry_point.load()
            for entry_point in pkg_resources.iter_entry_points('assnake.plugins')
        }
        for module_name, module_class in discovered_plugins.items():
            for wc_config in module_class.wc_configs:
                if wc_config is not None:
                    self.wc_config.update(wc_config)
                
        self.add_samples(fs_prefix, df, preproc, samples_to_add, do_not_add, pattern)

    def add_samples(self, fs_prefix, df, preproc, samples_to_add = [], do_not_add = [], pattern = ''):
        '''
        This function is used to add samples into the SampleSet.

        Args:
            fs_prefix: Prefix of the dataset on filesystem
            df: Name of the dataset
            preproc: Preprocessing you want to use
            samples_to_add: List of sample names to add
            do_not_add: list of sample names NOT to add
            pattern: sample names must match this glob pattern to be included. 
        '''
        samples = []
        fastq_gz_file_loc = self.wc_config['fastq_gz_file_wc'].format(
            fs_prefix=fs_prefix, df=df, preproc=preproc, 
            strand='R1',sample = '*')
        fs_names = [f.split('/')[-1].split('.')[0].replace('_R1', '') for f in glob.glob(fastq_gz_file_loc)]

        if pattern != '':
            fs_names = [f.split('/')[-1] for f in 
            glob.glob(self.wc_config['sample_dir_wc'].format(fs_prefix=fs_prefix, df=df, preproc=preproc, sample = pattern))]

        sample_dir_wc = self.wc_config['sample_dir_wc']
        fastq_gz_file_wc = self.wc_config['fastq_gz_file_wc']
        count_wc = self.wc_config['count_wc']

        fs_names = list(set(fs_names) - set(do_not_add))
        if len(samples_to_add) > 0:
            fs_names = fs_names and samples_to_add

        samples = [loaders.load_sample(fs_prefix, df, preproc, fs_name,
                        sample_dir_wc = sample_dir_wc, fastq_gz_file_wc = fastq_gz_file_wc, 
                        count_wc=count_wc) for fs_name in fs_names]
        
        samples_pd = pd.DataFrame(samples)
        # samples_pd.index = samples_pd['fs_name'] + ':' + samples_pd['preproc'] # We can debate on this

        self.samples_pd = pd.concat([self.samples_pd, samples_pd], sort=True)


    

    


from assnake.core.sample_set import SampleSet
import os, glob, yaml, time
import pandas as pd
from assnake.api.loaders import load_df_from_db, load_sample
from assnake.utils import load_config_file
from assnake.viz import plot_reads_count_change
import click

class Dataset:

    df = '' # name on file system
    fs_prefix = '' # prefix on file_system
    full_path = ''
    sample_sets = {} # Dict of sample sets, one for each preprocessing

    sources = None
    biospecimens = None
    mg_samples = None


    def __init__(self, df):
        config = load_config_file()
        info = load_df_from_db(df, include_preprocs = True)

        self.df =  info['df']
        self.fs_prefix =  info['fs_prefix']
        self.full_path = os.path.join(self.fs_prefix, self.df)

        preprocs = info['preprocs']
        preprocessing = {}
        for p in preprocs:
            samples = SampleSet(self.fs_prefix, self.df, p)
            if len(samples.samples_pd) > 0:
                samples = samples.samples_pd[['preproc', 'df', 'fs_prefix', 'fs_name', 'reads']]
                preprocessing.update({p:samples})
            

        self.sample_sets = preprocessing
        self.sample_containers = pd.concat(self.sample_sets.values())
        self.self_reads_info = self.sample_containers.pivot(index='fs_name', columns='preproc', values='reads')
  

    def plot_reads_loss(self, preprocs = [], sort = 'raw'):
        # preprocs = list(self.self_reads_info.columns)
        plot_reads_count_change(self.self_reads_info[preprocs].copy(), preprocs = preprocs, sort = sort, plot=True)

    def __str__(self):
        return self.df + '\n' + self.fs_prefix +'\n' + str(self.sample_sets)

    def __repr__(self):
        preprocessing_info = ''
        preprocs = list(self.sample_sets.keys())
        for preproc in preprocs:
            preprocessing_info = preprocessing_info + 'Samples in ' + preproc + ' - ' + str(len(self.sample_sets[preproc])) + '\n'
        return 'Dataset name: ' + self.df + '\n' + \
            'Filesystem prefix: ' + self.fs_prefix +'\n' + \
            'Full path: ' + os.path.join(self.fs_prefix, self.df) + '\n' + preprocessing_info

    def to_dict(self):
        preprocs = {}
        for ss in self.sample_sets:
            preprocs.update({ss : self.sample_sets[ss].to_dict(orient='records')})
        return {
            'df': self.df,
            'fs_prefix': self.fs_prefix,
            'preprocs': preprocs
        }



class SnakeModule:
    name = ''
    install_dir = ''
    snakefiles = []
    invocation_commands = []
    initialization_commands = []
    wc_configs = []
    initialization_commands = []
    results = []

    def __init__(self, name, install_dir, snakefiles, invocation_commands, initialization_commands = [], wc_configs = [], results = []):

        self.name = name
        self.install_dir = install_dir
        self.snakefiles = snakefiles
        self.invocation_commands = invocation_commands
        self.initialization_commands = initialization_commands
        self.wc_configs = wc_configs
        self.results = results


import os, click, glob
from assnake.cli.cli_utils import generic_command_individual_samples, generate_result_list, sample_set_construction_options, add_options
from assnake.utils import read_yaml

class Result:

    name = '' # This name will be used as command name and in web-service
    result_wc = '' # Just ONE wc that will be populated with data and passed to snakemake, so it can understand what it needs to do
    input_type = '' # sample_file, sample, sample_set ...
    additional_inputs = '' # db, fasta_ref ...
    git_results = [] # List of files that will be added to git repo
    workflows = [] # List of files with snakemake workflow files (.smk)

    imvocation_command = None

    def __init__(self, name, workflows, result_wc, input_type, additional_inputs, invocation_command = None, wc_config = None): 
        self.name = name
        self.result_wc = result_wc
        self.input_type = input_type
        self.additional_inputs = additional_inputs
        self.workflows = workflows
        self.wc_config = wc_config

        if invocation_command == None:
            if self.input_type == 'illumina_strand_file':
                @click.command(self.name, short_help=self.name)
                @add_options(sample_set_construction_options)
                @click.pass_obj
                def result_invocation(config, **kwargs):
                    kwargs.update({'strand': 'R1'})
                    sample_set, sample_set_name = generic_command_individual_samples(config,  **kwargs)
                    config['requests'] += generate_result_list(sample_set, self.result_wc, **kwargs)

                self.invocation_command = result_invocation
            elif self.input_type == 'illumina_sample':
                @click.command(self.name + '_fff', short_help=self.name)
                @add_options(sample_set_construction_options)
                @click.pass_obj
                def trimmomatic_invocation(config, params, **kwargs):
                    kwargs.update({'params': params})
                    sample_set, sample_set_name = generic_command_individual_samples(config,  **kwargs)
                    config['requests'] += generate_result_list(sample_set, self.result_wc, **kwargs)
        elif callable(invocation_command):
            self.invocation_command = invocation_command

    @classmethod
    def from_location(cls, name, location, input_type, additional_inputs = None, invocation_command = None):

        # Try to find all needed files for Result
        workflows = glob.glob(os.path.join(location, 'workflow*.smk'))
        wc_config = os.path.join(location, 'wc_config.yaml')

        if len(workflows) > 0:
            [print(w) for w in workflows]
        else:
            print('===', name)
            print('No workflow files found')

        if os.path.isfile(wc_config):
            wc_config = read_yaml(wc_config)
        else:
            print('No config file found')
            exit()

        x = cls(
            name = name, 
            workflows = workflows,
            result_wc = wc_config[name + '_wc'],
            input_type =  input_type,
            additional_inputs = additional_inputs,
            invocation_command = invocation_command,
            wc_config=wc_config
        )

        return x

    def generate_result_list(self, input, additional_inputs = None):
        res_list = []

        return res_list

    


import os
import glob

import pandas as pd
import numpy as np
import yaml
import assnake
import assnake.utils

from assnake.utils import bytes2human

def load_count(fs_prefix, df, preproc, sample, report_bps=False, verbose=False, count_wc=''):
    """
    Loads information about read and bp count in paired-end sample.
    """
    strands = ['R1', 'R2']
    
    count_loc1 = count_wc.format(fs_prefix=fs_prefix, df=df, preproc=preproc, sample=sample, strand=strands[0])
    count_loc2 = count_wc.format(fs_prefix=fs_prefix, df=df, preproc=preproc, sample=sample, strand=strands[1])
    
    count_dict = {'reads': -1}

    try:
        with open(count_loc1, 'r') as f:
            line = f.readline().rstrip()
            count_dict['reads'] += int(line.split(' ')[0]) + 1
            if report_bps:
                count_dict.update({'bps':int(line.split(' ')[1])})
        # with open(count_loc2, 'r') as f:
        #     line = f.readline().rstrip()
        #     count_dict['reads'] += int(line.split(' ')[0])
        #     if report_bps:
        #         count_dict['bps'] += int(line.split(' ')[1])
    except:
        if verbose: 
            print('error loading counts: ', sample)
        if report_bps:
            count_dict.update({'bps': -1})
        return count_dict
    
    return count_dict
        

def load_dfs_from_db(db_loc):
    """
    Returns dict of dictionaries with info about datasets from fs database. Key - df name
    Mandatory fields: df, prefix
    """
    dfs = {}
    curr_dir = os.path.dirname(os.path.abspath(__file__))
    config = assnake.utils.load_config_file()
    df_info_locs = glob.glob(config['assnake_db']+'/datasets/*/df_info.yaml')
    
    for df_info in df_info_locs:
        with open(df_info, 'r') as stream:
            try:
                info = yaml.load(stream, Loader=yaml.FullLoader)
                if 'df' in info:
                    dfs.update({info['df']: info})
            except yaml.YAMLError as exc:
                print(exc)
    return dfs

def load_df_from_db(df_name, db_loc='', include_preprocs = False):
    """
    Returns one dictionary with df info
    """
    config = assnake.utils.load_config_file()

    df_info_loc = config['assnake_db']+'/datasets/{df}/df_info.yaml'.format(df = df_name)
    df_info = {}
    with open(df_info_loc, 'r') as stream:
        try:
            info = yaml.load(stream, Loader=yaml.FullLoader)
            if 'df' in info:
                df_info =  info
        except yaml.YAMLError as exc:
            print(exc)

    reads_dir = os.path.join(df_info['fs_prefix'], df_info['df'], 'reads/*')
    preprocs = [p.split('/')[-1] for p in glob.glob(reads_dir)]
    preprocessing = {}
    if include_preprocs:
        for p in preprocs:
            samples = assnake.core.sample_set.SampleSet(df_info['fs_prefix'], df_info['df'], p)
            samples = samples.samples_pd[['fs_name', 'reads']].to_dict(orient='records')
            preprocessing.update({p:samples})
    df_info.update({"preprocs": preprocessing})
    return df_info

def load_sample(fs_prefix, df, preproc, sample,
                report_bps=False, report_size=False, verbose=False,
                sample_dir_wc = '', fastq_gz_file_wc = '', count_wc=''):
    '''
    Loads all necessary info about given sample from file system.
    '''
    # Init start values
    sample_dict = {}
    strands = ['R1', 'R2']
    
    final_preproc = ''
    size = 0
    containers = []
    preprocs = []


    # Now select what preprocessing we want to use
    if preproc == 'longest':
        preprocs = [p.split('/')[-2] for p in 
                    glob.glob(fastq_gz_file_wc.format(fs_prefix=fs_prefix, df=df, preproc='*', sample=sample, strand='R1'))]
    else:
        preprocs = [p.split('/')[-2] for p in 
                    glob.glob(fastq_gz_file_wc.format(fs_prefix=fs_prefix, df=df, preproc=preproc, sample=sample, strand='R1'))]
    for p in preprocs:
        r1 = fastq_gz_file_wc.format(fs_prefix=fs_prefix, df=df, preproc=p, sample=sample, strand=strands[0])
        r2 = fastq_gz_file_wc.format(fs_prefix=fs_prefix, df=df, preproc=p, sample=sample, strand=strands[1])

        if os.path.isfile(r1) and os.path.isfile(r2):
            containers.append(p)
            if len(p) > len(final_preproc):
                final_preproc = p
                if report_size:
                    size = os.path.getsize(r1) + os.path.getsize(r2)
                    sample_dict.update({'size': bytes2human(size, symbols='iec'), 'bytes': size})
    return {'df':df, 
            'fs_name':sample, 
            'sample':sample,  
            'preproc':final_preproc, 
            'fs_prefix': fs_prefix,
            #'preprocs':containers, 
            **load_count(fs_prefix, df, final_preproc, sample, verbose, count_wc=count_wc)}


def load_sample_set(wc_config, fs_prefix, df, preproc, samples_to_add = [], do_not_add = [], pattern = '*'):
    '''
    This function is used to add samples into the SampleSet.

    Args:
        fs_prefix: Prefix of the dataset on filesystem
        df: Name of the dataset
        preproc: Preprocessing you want to use
        samples_to_add: List of sample names to add
        do_not_add: list of sample names NOT to add
        pattern: sample names must match this glob pattern to be included. 
    '''
    samples = []
    fastq_gz_file_loc = wc_config['fastq_gz_file_wc'].format(
        fs_prefix=fs_prefix, df=df, preproc=preproc, 
        strand='R1', sample = pattern)
    
    fs_names = [f.split('/')[-1].split('.')[0].replace('_R1', '') for f in glob.glob(fastq_gz_file_loc)]

    sample_dir_wc    = wc_config['sample_dir_wc']
    fastq_gz_file_wc = wc_config['fastq_gz_file_wc']
    count_wc         = wc_config['count_wc']

    fs_names = list(set(fs_names) - set(do_not_add))

    if len(samples_to_add) > 0: fs_names = fs_names and samples_to_add

    samples = [load_sample(fs_prefix, df, preproc, fs_name,
                    sample_dir_wc = sample_dir_wc, fastq_gz_file_wc = fastq_gz_file_wc, 
                    count_wc=count_wc) for fs_name in fs_names]
    
    sample_set = pd.DataFrame(samples)
    return sample_set

fields = ['sample', 'sequencing_run']


def update_fs_samples_csv(dataset):
    '''
    Scans dataset folder and updates fs_samples.tsv if necessary
    
    :param dataset: Name of the dataset
    :return: Returns sample dict in loc
    
    '''
    fs_samples_tsv_loc = '{config}/datasets/{df}/fs_samples.tsv'.format(config=assnake.utils.load_config_file()['assnake_db'], df=dataset)
    df = assnake.Dataset(dataset)
    fs_samples_pd = pd.concat(list(df.sample_sets.values()))
    fs_samples_pd = df.sample_sets['raw']
    fs_samples_pd['final_preprocessing'] = 'never_set'
    fs_samples_pd.to_csv(fs_samples_tsv_loc, sep='\t', index = False)

    return True

def samples_to_pd(samples):
    meta_df = pd.DataFrame(columns=['fs_name', 'df', 'preproc', 'size', 'bytes', 'reads','bps'])
    for s in samples:
        meta_df.loc[s['sample']]=[s['fs_name'], s['df'], s['preproc'], s['size'], s['bytes'], s['reads'],s['bps']]
    return meta_df.sort_values('fs_name')






import click, sys, os, glob, yaml, shutil
import pandas as pd
import assnake.api.loaders
import assnake.core.sample_set
from tabulate import tabulate
from assnake.api import fs_helpers
from assnake.utils import pathizer, dict_norm_print, download_from_url
from zipfile import ZipFile
from assnake.api.loaders import update_fs_samples_csv

# some util
def show_av_dict(dfs):
    '''
    print out available datasets from dict of db's (assnake.api.loaders.load_dfs_from_db(''))
    '''
    avail_dfs = ''
    for i, item in enumerate(dfs.keys()):
        avail_dfs += '{}. {} \t'.format(i + 1, item)

    if avail_dfs == '':
        avail_dfs = 'NA'
    click.echo('Available: ' + avail_dfs)
    exit(2)


# ---------------------------------------------------------------------------------------
#                                     LIST
# ---------------------------------------------------------------------------------------
@click.command(name='list')
def df_list():
    """List datasets in database"""
    dfs = assnake.api.loaders.load_dfs_from_db('')
    if len(list(dfs.keys())) == 0:
        click.echo('No datasets in your system yet!\nYou can create one by running\n' +
                   click.style('  assnake dataset create  ', bg='blue', fg='white', bold=True))

    for df in dfs.values():
        df_name = df['df']
        click.echo(click.style('' * 2 + df_name + ' ' * 2, fg='green', bold=True))
        click.echo('  Filesystem prefix: ' + df.get('fs_prefix', ''))
        click.echo('  Full path: ' + os.path.join(df.get('fs_prefix', ''), df['df']))
        click.echo('  Description: ')
        dict_norm_print(df.get('description', ''))
        click.echo('')


# ---------------------------------------------------------------------------------------
#                                   CREATE
# ---------------------------------------------------------------------------------------

def check_df_name_avialability(df):
    
    dfs_in_db = assnake.api.loaders.load_dfs_from_db('')
    if df in dfs_in_db:
        click.secho('Dataset with such name already exists!. Aborting.')
        exit()
    return df

def validate_df_prefix(df, fs_prefix, create_prefix = False, allow_not_empty_full_path = False):
    """
    Ensures that can write to full_df_path and it is empty
    """
    df = check_df_name_avialability(df)
    if df is None:
        exit()

    if not os.path.isabs(fs_prefix):
        click.secho('fs_prefix is not absolute!') 
        exit()
    full_df_path = os.path.join(fs_prefix, df)

    if os.path.isdir(full_df_path):
        if not os.listdir(full_df_path) and os.access(full_df_path, os.W_OK):
            click.echo('Great! ' + full_df_path + ' exists, is empty and assnake can write to that folder, initializing dataset here')
            return full_df_path
        elif os.listdir(full_df_path):
            if allow_not_empty_full_path:
                click.echo(full_df_path + ' exists, but it is not empty. allow_not_empty_full_path is true, existance logic will be handled downstream.')
                return full_df_path
            else:
                click.echo('Oops =( ' + full_df_path + ' exists, but it is not empty. Aborting.')
                return None
        elif not os.access(full_df_path, os.W_OK):
            click.echo('Oops =( ' + full_df_path + ' exists, but assnake doesnt have permissions to write there. Aborting.')
            return None
    else:
        click.echo('Oops =( ' + full_df_path + ' exists, but it is not directory. Aborting.')

        
    if os.path.isdir(fs_prefix):
        if os.access(fs_prefix, os.W_OK):
            click.echo('Great! ' + fs_prefix + ' exists, and assnake can write to that folder, initializing dataset in:')
            click.echo(full_df_path + '  (Will be created)')
            return full_df_path
    else:
        if not create_prefix:
            click.echo('Oops =( ' + fs_prefix + ' doesnt exist, and create_prefix is set to False. Aborting.')
        else:
            click.echo('Great! ' + fs_prefix + ' will be created, and dataset will be initialized in:')
            click.echo(full_df_path + '  (Will be created)')
            return full_df_path

    return None

def check_absolute_path(path, path_name):
    path = path.rstrip('\/')
    if not os.path.isabs(path):
        click.secho(path_name + ' is not absolute!') 
        exit()
    return path

@click.command(name='create')

@click.option('--df-name', '-d', help='Name of the dataset', required = False)

@click.option('--data-storage-folder', '-f', 
    help='Folder where you want to store your data. \
        Directory with provided df-name will be created inside this folder. MUST exist, may be not empty.\
        Resulting {storage-folder}/{df-name} directory MUST be empty, may not exist. \
        For regestiring existing datasets inside new Assnake instance use assnake dataset register', required=False)

@click.option('--full-path-to-df', '-p', 
    help='Full path to dataset folder. \
    Last part of the path (basename) will be used as df-name. MUST be empty, may not exist. \
    For regestiring existing datasets inside new Assnake instance use assnake dataset register', required=False)


@click.option('--first-preprocessing-name', 
    help='Name of your first preprocessing. raw by default. You want to set it to sra, for exaple if yoo are planning to download from NCBI. Purely cosmetic effect.', required=False, default = 'raw')

@click.option('--description', '-D', nargs=2, multiple=True, required=False, type=click.Tuple([str, str]),
              help='Add some description in this way ` assnake dataset create ... -D property_1 value_1 ... -D property_n value_n`')

@click.option('--quietly', '-q', is_flag=True, help='Doing it quietly. No questions.')
@click.option('--test-data', '-t', is_flag=True, help='Download test data from Humann2 tutorial')
@click.pass_obj
def df_create(config, df_name, data_storage_folder, full_path_to_df,first_preprocessing_name, description, quietly, test_data):
    """Register your dataset inside ASSNAKE!\n
        You can use it in interactive mode.
        Usage: assnake dataset create [dataset] or -d [dataset] ..

    """
    
    if ((df_name is None) or (data_storage_folder is None)):
        if full_path_to_df is None:
            click.echo('You must specify df_name AND data_storage_folder, OR full_path_to_df')
            exit()
        
        else: # full_path_to_df road
            # click.echo('df-name or data-storage-folder not set, using full_path_to_df')
            click.echo('Using full_path_to_df')
            full_path_to_df = check_absolute_path(full_path_to_df, 'full_path_to_df')
            df = os.path.basename(full_path_to_df)
            fs_prefix = os.path.dirname(full_path_to_df)
            full_df_path = validate_df_prefix(df, fs_prefix, True)

    # df_name AND data_storage_folder road
    elif (df_name is not None) and (data_storage_folder is not None):
        if full_path_to_df is not None:
            click.echo('df-name AND data-storage-folder are set, ignoring full_path_to_df')

        click.secho('Using df_name AND data_storage_folder')
        data_storage_folder = check_absolute_path(data_storage_folder, 'data_storage_folder')
        df = df_name # TODO VALIDATE
        fs_prefix = data_storage_folder
        full_df_path = validate_df_prefix(df, fs_prefix, True)

    os.makedirs(full_df_path, exist_ok=True)
    os.makedirs(os.path.join(full_df_path, 'reads', first_preprocessing_name), exist_ok=True)
    df_info = {'df': df, 'fs_prefix': fs_prefix, 'description': {}}

    assnake_db = config['config']['assnake_db']
    os.makedirs(os.path.join(assnake_db, 'datasets'), exist_ok=True)
    df_path_in_assnake = os.path.join(assnake_db, 'datasets', df)
    os.symlink(full_df_path, df_path_in_assnake, target_is_directory = True)

    with open(os.path.join(df_path_in_assnake, 'df_info.yaml'), 'w') as info_file:
        yaml.dump(df_info, info_file, default_flow_style=False)
    click.secho('Saved dataset ' + df + ' sucessfully!', fg='green')


    if test_data:
        download_from_url('http://kronos.pharmacology.dal.ca/public_files/tutorial_datasets/mgs_tutorial_Oct2017.zip', 
                                os.path.join(fs_prefix, df,'mgs_tutorial_Oct2017.zip'))
        
        with ZipFile(os.path.join(fs_prefix, df,'mgs_tutorial_Oct2017.zip'), 'r') as zipObj:
            
            zipObj.extractall(os.path.join(fs_prefix, df,'./'))
        shutil.rmtree(os.path.join(fs_prefix, df, 'reads'), ignore_errors=True)
        os.makedirs(os.path.join(fs_prefix, df, 'reads'), exist_ok=True)
        shutil.move (os.path.join(fs_prefix, df,'mgs_tutorial_Oct2017/raw_data'), os.path.join(fs_prefix, df, 'reads/raw'))
        # TODO cleanup
        shutil.rmtree(os.path.join(fs_prefix, df,'mgs_tutorial_Oct2017'), ignore_errors=True)

# ---------------------------------------------------------------------------------------
#                                   INIT
# ---------------------------------------------------------------------------------------

@click.command(name='init', help = 'Register dataset in Assnake based on the folder from where you called the command. (Working directory)')
@click.option('--df-name', '-d', help='Name of the dataset. If provided, folder with this name will be created in current dir.', required = False)
@click.option('--first-preprocessing-name', 
    help='Name of your first preprocessing. raw by default. You want to set it to sra, for exaple if yoo are planning to download from NCBI. Purely cosmetic effect.', required=False, default = 'raw')

@click.pass_obj
def df_init(config, df_name, first_preprocessing_name):
    cwd = os.getcwd()

    if df_name is None:
        df = os.path.basename(cwd)
        fs_prefix = os.path.dirname(cwd)
        full_df_path = validate_df_prefix(df, fs_prefix, True, True)
    else:
        df = df_name
        fs_prefix = cwd
        full_df_path = validate_df_prefix(df, fs_prefix, True, True)

    full_path_empty = True
    if os.path.isdir(full_df_path) and not os.listdir(full_df_path):
        click.echo(full_df_path + ' not empty!')
        click.echo('Trying to import as existing Assnake dataset (Not properly implemented)')
        full_path_empty = False
 

    os.makedirs(os.path.join(full_df_path, 'reads', first_preprocessing_name), exist_ok=True)
    df_info = {'df': df, 'fs_prefix': fs_prefix, 'description': {}}

    assnake_db = config['config']['assnake_db']
    os.makedirs(os.path.join(assnake_db, 'datasets'), exist_ok=True)
    df_path_in_assnake = os.path.join(assnake_db, 'datasets', df)
    os.symlink(full_df_path, df_path_in_assnake, target_is_directory = True)

    df_info_loc = os.path.join(df_path_in_assnake, 'df_info.yaml')
    if not os.path.isfile(df_info_loc):
        with open(df_info_loc, 'w') as info_file:
            yaml.dump(df_info, info_file, default_flow_style=False)
    click.secho('Saved dataset ' + df + ' sucessfully!', fg='green')

# ---------------------------------------------------------------------------------------
#                                   INFO
# ---------------------------------------------------------------------------------------
@click.command(name='info')
@click.option('--df', '-d', help='Name of the dataset', required=False)
@click.option('--preproc', '-p', help='Show samples for preprocessing', required=False)
@click.argument('df_arg', required=True)
@click.pass_obj
def df_info(config, df, preproc, df_arg):
    """View info for the specific dataset
        Usage: assnake dataset info [dataset] or -d [dataset] ...

    """
    if not (bool(df is None) ^ bool(df_arg is None)):
        click.echo('Please, specify dataset either as option or argument')
        df = click.prompt('Type the name in')
    if df is None:
        df = df_arg
    dfs = assnake.api.loaders.load_dfs_from_db('')
    try:
        df_info = dfs[df]
    except KeyError as e:
        click.echo('Can`t reach database with such name')
        show_av_dict(dfs)
    click.echo(click.style('' * 2 + df_info['df'] + ' ' * 2, fg='green', bold=True))
    click.echo('Filesystem prefix: ' + df_info.get('fs_prefix', ''))
    click.echo('Full path: ' + os.path.join(df_info.get('fs_prefix', ''), df))
    click.echo('Description: ')
    dict_norm_print(df_info.get('description', ''))

    mg_samples_loc = os.path.join(config['config']['assnake_db'], 'datasets', df_info['df'], 'mg_samples.tsv')

    if os.path.isfile(mg_samples_loc):
        click.echo(tabulate(pd.read_csv(mg_samples_loc, sep='\t'), headers='keys', tablefmt='fancy_grid'))

    reads_dir = os.path.join(df_info['fs_prefix'], df_info['df'], 'reads/*')
    preprocs = [p.split('/')[-1] for p in glob.glob(reads_dir)]
    preprocs.sort()
    preprocessing = {}
    all_samples = []
    for p in preprocs:
        samples = assnake.core.sample_set.SampleSet(df_info['fs_prefix'], df_info['df'], p)
        # samples.add_samples(df_info['fs_prefix'], df_info['df'], p)
        all_samples += (list(samples.samples_pd['fs_name']))
        samples = samples.samples_pd[['fs_name', 'reads']].to_dict(orient='records')
        # click.secho(p + ' ' + str(len(samples)) + ' samples')
        preprocessing.update({p: samples})

    click.echo('\nTotal samples: ' + str(len(set(all_samples))) + '\n')

    for key, value in preprocessing.items():
        click.echo('Samples in ' + click.style(key, bold=True) + ': ' + str(len(value)))
    if preproc is not None:
        samples_pd = pd.DataFrame(preprocessing[preproc])
        # print(samples_pd)
        click.echo(tabulate(samples_pd.sort_values('reads'), headers='keys', tablefmt='fancy_grid'))


# ---------------------------------------------------------------------------------------
#                                   DELETE
# ---------------------------------------------------------------------------------------
@click.command(name='delete')
@click.option('--df', '-d',
              help='Name of dataset to delete.', type=click.STRING )
@click.option('--hard', help='If is set, hard removing will be used instead of modifying congig file', is_flag=True)
@click.argument('df_arg', required=False)
@click.pass_obj
def df_delete(config, df, hard, df_arg):
    """
    Delete datasets
    Usage: assnake dataset delete [dataset] or -d [dataset] ...
    """
    if not (bool(df is None) ^ bool(df_arg is None)):
        click.echo('Please, specify dataset either as option or argument')
        df = click.prompt('Type the name in:')
    if df is None:
        df = df_arg
    dfs = assnake.api.loaders.load_dfs_from_db('')
    try:
        df_info = dfs[df]
    except KeyError as e:
        click.echo('Can`t reach database with such name')
        show_av_dict(dfs)

    respond = fs_helpers.delete_ds(df)
    if respond[0]:
        click.secho('Successfully deleted', fg='bright_white', bg='green')
    else:
        click.secho('ERROR', bg='red')
        click.echo('For details see traceback below')
        click.echo(respond[1])

    if hard and click.confirm(
        'Are you sure to delete this nice and probably huge datasets, which you may redownload for eternity -- use modifying config instead?',
        abort=True):
        shutil.rmtree(os.path.join(df_info.get('fs_prefix', ''), df))

# ---------------------------------------------------------------------------------------
#                                   IMPORT-READS
# ---------------------------------------------------------------------------------------
# DONE decide if we need either d and t or proceed both arguments as one and automatically choose path or not
@click.command(name='import-reads')
@click.option('--reads', '-r', prompt='Location of folder with read files',
              help='Location of folder with read files', type=click.Path())
@click.option('--dataset', '-d', help='Assnake dataset name. If -t is not specified', required=False)
@click.option('--rename-method', help='How to rename samples', type=click.Choice(['replace-', 'removeSending'], case_sensitive=False), required=False)
@click.option('--target', '-t', help='Location of the target directory. If -d is not specified.', required=False,
              type=click.Path())
@click.option('--sample_set', '-s', help='Comma-divided list of samples of interest', required=False)
@click.option('--sample_list', '-l', help='Location of file with line by line samples of interest', required=False,
              type=click.Path())
@click.option('--copy', help='If is set, hard copying will be used instead of symbolic links ', is_flag=True)
@click.pass_obj
def df_import_reads(config, reads, dataset, rename_method, target, sample_set, sample_list, copy):
    """
    Import reads from directory to assnake dataset. Currently local text files are supported. The --target argument
    point to location (relative or absolute) of assnake dataset in your file system. Please, pay attention,
    that -t and -d  arguments are excclusive for each over -- specify only one of them -- as well as -s and -l.
    With -s `sample_1,sample_2,...,sample_n` notation is implied (no whitespaces between sample names)
    """
    # print(os.getcwd())
    # This is cli wrapping over create_links from fs_helpers

    # stuff about presence of arguments
    arg_d = not bool(dataset is None)
    arg_t = not bool(target is None)
    arg_s = not bool(sample_set is None)
    arg_l = not bool(sample_list is None)

    # check if samples arguments are ok
    if arg_l & arg_s:
        click.secho('Collision tends to be observed. Please, specify either list of samples in prompt or in file',
                    err=True)
        exit(1)

    # check if destination args are ok
    if not (arg_d ^ arg_t):
        click.secho('Please, specify either database (-d) or absolute path (-t)', err=True)
        exit(1)

    # some stuffff to ensure correctness of source and destination (how philosophical)
    if arg_d:
        try:
            df_info = assnake.api.loaders.load_df_from_db(dataset)
        except Exception as e:
            dfs = assnake.api.loaders.load_dfs_from_db('')
            click.echo('Can`t reach database with such name', err=True)
            show_av_dict(dfs)
        target = '{}/{}/reads/raw'.format(df_info['fs_prefix'], df_info['df'])
    else:
        target = pathizer(target)
        if not os.path.exists(target):
            click.secho("Provided sample-list file couldn't be detected", err=True)
            exit(2)
    target = '{}/{}/reads/raw'.format(df_info['fs_prefix'], df_info['df'])
    os.makedirs(target, exist_ok=True)
    # def rename(sample):
    #     new_name_wc = 'Rst{}_{}_B2'
    #     splitted = sample.split('_')
    #     splitted[2] = (1 if len(splitted) == 3 else a)
    #     return new_name_wc.format(splitted[1], splitted[2])

    if rename_method == 'removeSending':
        modify_name=lambda arg: '_'.join(arg.replace('-', '_').split('_')[0:-1])
    else:
        modify_name=lambda arg: arg.replace('-', '_')

    samples_in_run = fs_helpers.get_samples_from_dir(reads, modify_name)
    samples_in_run['fs_name'] = samples_in_run['modified_name']
    fs_helpers.create_links(target,  samples_in_run, hard=copy)

    update_fs_samples_csv(df_info['df'])
    click.secho("SUCCESSFULLY IMPORTED READS!", bg='green') 
    # sample_names = {d['sample_name'] for d in dicts}
    # if arg_s:
    #     samples_of_interest = sample_set.split(',')
    # elif arg_l:
    #     sample_list = pathizer(sample_list)
    #     if os.path.exists(sample_list):
    #         with open(sample_list, 'r') as ls_file:
    #             samples_of_interest = ls_file.readlines()
    #     else:
    #         click.secho("Provided sample-list file couldn't be detected", err=True)
    # else:
    #     samples_of_interest = sample_names

    # # Such a mess! check if specified samples are in directory
    # if (arg_s or arg_l) and (len(set(samples_of_interest) - {d['sample_name'] for d in dicts}) != 0):
    #     click.secho("Warning! Samples, been specified, are not in reads file", err=True)
    #     click.confirm('Continue?', abort=True)
    # if copy:
    #     click.secho("Please, keep in mind, that hard copying may take plenty of time")

    # Here the logic -- just call create_links
    


@click.command(name='rescan')
@click.option('--dataset', '-d', help='Assnake dataset name', required=False)
@click.argument('df_arg', required=False)
@click.pass_obj
def rescan_dataset(config, dataset, df_arg):
    """
    Now it just updates fs_samples.tsv in ./assnkae_db/{dataset}/

    Usage: assnake dataset rescan [dataset] or -d [dataset] ..
    """
    if not (bool(dataset is None) ^ bool(df_arg is None)):
        click.echo('Please, specify dataset either as option or argument')
        dataset = click.prompt('Type the name in:')
    if dataset is None:
        dataset = df_arg
    success = update_fs_samples_csv(dataset)
    if success:
        click.secho('SUCCESSFULLY UPDATED INFORMATION IN DATABASE!', fg='green')



from setuptools import setup, find_packages
import fastentrypoints
import os
import configparser

# Creating config_internal.ini
dir_of_this_file = os.path.dirname(os.path.abspath(__file__))
config_internal = configparser.ConfigParser()
config_internal['GENERAL'] = {'config_loc': 'None'}
with open(os.path.join(dir_of_this_file, './assnake/config_internal.ini'), 'w+') as configfile:
    config_internal.write(configfile)

setup(name='assnake', 
    version='0.8.9.1',
    license='MIT',        
    description = 'System for metagenomics data analysis',   
    author = 'Dmitry Fedorov',                  
    author_email = 'fedorov.de@gmail.com',      
    url = 'https://github.com/ASSNAKE/assnake',   
    download_url = 'https://github.com/ASSNAKE/assnake/archive/v0.8.8.tar.gz',    # I explain this later on
    keywords = ['ILLUMINA', 'NGS', 'METAGENOMIC', 'DATA'], 
    packages=['assnake'],
    install_requires=[
        'numpy', 'Click', 'pyyaml>=5', 'pandas', 
        'tabulate', 'snakemake', 'drmaa', 
        'parse', 'pycallgraph', 'tqdm', 'scipy', 'plotly', 'matplotlib'
    ],
    entry_points='''
        [console_scripts]
        assnake=assnake.cli.assnake_cli:main
    ''',

    classifiers=[
        'Development Status :: 2 - Pre-Alpha',      # Chose either "3 - Alpha", "4 - Beta" or "5 - Production/Stable" as the current state of your package
        'Intended Audience :: Science/Research',      # Define that your audience are developers
        'Topic :: Scientific/Engineering :: Bio-Informatics',
        'License :: OSI Approved :: MIT License',   # Again, pick a license
        'Programming Language :: Python :: 3.6',
        ],
    )


import argparse
import os.path
import os
import sys
import glob
import fnmatch
from shutil import copy2, rmtree
from assnake import utils
import traceback
import parse
from assnake.api.loaders import load_df_from_db
import pandas as pd



def get_samples_from_dir(directory_with_reads, modify_name = None):
    samples_list = []  # to write samples in
    
    ext = '.fastq.gz'  # extention
    ending_variants = [{'name': 'normal', 'strands': {'R1': '_R1', 'R2': '_R2'}},
                    # {'name': 'ILLUMINA_WITH_LANE', 'strands': {'R1': '_L001_R1_001', 'R2': '_L001_R2_001'}},
                    {'name': 'ILLUMINA_001', 'strands': {'R1': '_R1_001', 'R2': '_R2_001'}},
                    {'name': 'SRA', 'strands': {'R1': '_1', 'R2': '_2'}}]  # possible endigngs of files


    for ending_variant in ending_variants:

        variant_w_ext = ending_variant['strands']['R1'] + ext
        globbing_path = os.path.join(directory_with_reads, '*' + variant_w_ext)
        files_mathching_pattern = glob.glob(globbing_path)

        # get sample names
        samples = [
            filename.split('/')[-1].replace(variant_w_ext, '') # Take last part of path (basename) and replace _R1/fastq.gz with nothing to get sample name
            for filename in files_mathching_pattern
        ]
        
        # prepare sample dicts
        samples_list += [
            {
                'name_in_run': s,
                'modified_name': modify_name(s) if modify_name is not None else s,
                
                'ending_variant_id': ending_variant['name'],
                'ending_variant_R1': ending_variant['strands']['R1'],
                'ending_variant_R2': ending_variant['strands']['R2'],
                'directory': directory_with_reads,
                'extension': ext
            } 
            for s in samples
        ]
        

    return pd.DataFrame(samples_list)


def create_links(import_dir, samples, hard = False, create_dir_if_not_exist = False, rename = False, just_print = False):
    """
    This method creates links or hard copies reads files from one directory to another. 

    :param dir_with_reads: folder where to put reads
    :param original_dir: folders that contains original reads files 
    :param sample: dictionary from get_sample_dict_from_dir method
    :param hard: if hard copying is needed or symbolic is sufficient  (False)

    :returns:

    """
    orig_wc     = '{dir_w_reads}/{name_in_run}{ending_variant}{extension}'
    new_file_wc = '{import_dir}/{name_in_dataset}{strand}{extension}'

    # make dirs
    if not os.path.isdir(import_dir):
        if create_dir_if_not_exist:
            os.makedirs(import_dir)
        else:
            print("Such dir doesn't exist and create_dir_if_not_exist is set to False")

    for sample in samples.to_dict(orient = 'records'):
        
        src_r1 = orig_wc.format(
            dir_w_reads = sample['directory'], 
            name_in_run = sample['name_in_run'],
            ending_variant = sample['ending_variant_R1'],
            extension = sample['extension']
        )
        dst_r1 = new_file_wc.format(
            import_dir = import_dir, 
            name_in_dataset = sample['fs_name'],
            strand = '_R1',
            extension = sample['extension']
        )

        src_r2 = orig_wc.format(
            dir_w_reads = sample['directory'], 
            name_in_run = sample['name_in_run'],
            ending_variant = sample['ending_variant_R2'],
            extension = sample['extension']
        )
        dst_r2 = new_file_wc.format(
            import_dir = import_dir, 
            name_in_dataset = sample['fs_name'],
            strand = '_R2',
            extension = sample['extension']
        )

        if rename:
            os.rename(src_r1, dst_r1)
            os.rename(src_r2, dst_r2)
            return

        if hard:
            copy2(src_r1, dst_r1)
            copy2(src_r2, dst_r2)
            return
        # print('---')
        # print(src_r1, dst_r1)
        # print(src_r2, dst_r2)
        os.symlink(src_r1, dst_r1)
        os.symlink(src_r2, dst_r2)

def delete_ds(dataset):
    """
    Remove assnake dataset from database
    """
    try:
        os.remove(
            '{config}/datasets/{df}/df_info.yaml'.format(config=utils.load_config_file()['assnake_db'], df=dataset))
        return (True,)
    except Exception as e:
        return (False, traceback.format_exc())




import yaml, configparser, os, click
import os, sys
import requests, urllib
from tqdm import tqdm
import assnake.config_internal



def read_yaml(file_location):
    yaml_file = {}
    with open(file_location, 'r') as stream:
        try:
            yaml_file = yaml.load(stream, Loader=yaml.FullLoader)
            return yaml_file
        except yaml.YAMLError as exc:
            print(exc)


def get_internal_config():
    dir_of_this_file = os.path.dirname(os.path.abspath(__file__))
    config_internal = configparser.ConfigParser()

    config_internal.read(os.path.join(dir_of_this_file, './config_internal.ini'))
    

    print('===', assnake.config_internal.config_loc)



    return config_internal


def load_wc_config():
    dir_of_this_file = os.path.dirname(os.path.abspath(__file__))
    return (read_yaml(os.path.join(dir_of_this_file, './snake/wc_config.yaml')))


def load_config_file():
    config_loc = assnake.config_internal.config_loc
    return (read_yaml(config_loc))


def get_config_loc():
    config_internal = get_internal_config()
    return (assnake.config_internal.config_loc)


def check_if_assnake_is_initialized():
    config_loc = assnake.config_internal.config_loc
    if config_loc is None or not os.path.isfile(get_config_loc()):
        click.secho("You need to init your installation!", fg='red', bold=True)
        click.echo("Don't worry, it won't take long.")
        click.echo('Just run ' + click.style('assnake init start', bg='blue', fg='bright_white'))
        exit()

def update_config(dict_to_add):
    config = load_config_file()
    config.update(dict_to_add)
    config_location = get_config_loc()
    with open(config_location, 'w+') as file:
        _ = yaml.dump(config, file, sort_keys=False)




def pathizer(path):
    """
    Make from path absolute path
    :param path: absolute or relative path
    :return: absolute path
    """
    if path is None or path==' ' or path == '':
        return os.popen('pwd').read().replace('\n', '')
    final = '{prefix}/{rel_path}'.format(prefix=os.popen('pwd').read().replace('\n', ''), rel_path=path) if (path[0] != '/') else path
    return final.rstrip('\/')


def dict_norm_print(d, indent=1):
    if isinstance(d, str):
        click.echo('\t' * indent + str(d), nl=False)
    else:
        for key, value in d.items():
            click.echo('\t' * indent + str(key), nl=False)
            if isinstance(value, dict):
                click.echo('--↴	')
                dict_norm_print(value, indent + 1)
            else:
                click.echo('\t' * (indent) + str(value))


## {{{ http://code.activestate.com/recipes/578019/ (r15)
# see: http://goo.gl/kTQMs
SYMBOLS = {
    'customary': ('B', 'K', 'M', 'G', 'T', 'P', 'E', 'Z', 'Y'),
    'customary_ext': ('byte', 'kilo', 'mega', 'giga', 'tera', 'peta', 'exa',
                      'zetta', 'iotta'),
    'iec': ('Bi', 'Ki', 'Mi', 'Gi', 'Ti', 'Pi', 'Ei', 'Zi', 'Yi'),
    'iec_ext': ('byte', 'kibi', 'mebi', 'gibi', 'tebi', 'pebi', 'exbi',
                'zebi', 'yobi'),
}


def bytes2human(n, format='%(value).1f %(symbol)s', symbols='customary'):
    """
    Convert n bytes into a human readable string based on format.
    symbols can be either "customary", "customary_ext", "iec" or "iec_ext",
    see: http://goo.gl/kTQMs

      >>> bytes2human(0)
      '0.0 B'
      >>> bytes2human(0.9)
      '0.0 B'
      >>> bytes2human(1)
      '1.0 B'
      >>> bytes2human(1.9)
      '1.0 B'
      >>> bytes2human(1024)
      '1.0 K'
      >>> bytes2human(1048576)
      '1.0 M'
      >>> bytes2human(1099511627776127398123789121)
      '909.5 Y'

      >>> bytes2human(9856, symbols="customary")
      '9.6 K'
      >>> bytes2human(9856, symbols="customary_ext")
      '9.6 kilo'
      >>> bytes2human(9856, symbols="iec")
      '9.6 Ki'
      >>> bytes2human(9856, symbols="iec_ext")
      '9.6 kibi'

      >>> bytes2human(10000, "%(value).1f %(symbol)s/sec")
      '9.8 K/sec'

      >>> # precision can be adjusted by playing with %f operator
      >>> bytes2human(10000, format="%(value).5f %(symbol)s")
      '9.76562 K'
    """
    n = int(n)
    if n < 0:
        raise ValueError("n < 0")
    symbols = SYMBOLS[symbols]
    prefix = {}
    for i, s in enumerate(symbols[1:]):
        prefix[s] = 1 << (i + 1) * 10
    for symbol in reversed(symbols[1:]):
        if n >= prefix[symbol]:
            value = float(n) / prefix[symbol]
            return format % locals()
    return format % dict(symbol=symbols[0], value=n)


def human2bytes(s):
    """
    Attempts to guess the string format based on default symbols
    set and return the corresponding bytes as an integer.
    When unable to recognize the format ValueError is raised.

      >>> human2bytes('0 B')
      0
      >>> human2bytes('1 K')
      1024
      >>> human2bytes('1 M')
      1048576
      >>> human2bytes('1 Gi')
      1073741824
      >>> human2bytes('1 tera')
      1099511627776

      >>> human2bytes('0.5kilo')
      512
      >>> human2bytes('0.1  byte')
      0
      >>> human2bytes('1 k')  # k is an alias for K
      1024
      >>> human2bytes('12 foo')
      Traceback (most recent call last):
          ...
      ValueError: can't interpret '12 foo'
    """
    init = s
    num = ""
    while s and s[0:1].isdigit() or s[0:1] == '.':
        num += s[0]
        s = s[1:]
    num = float(num)
    letter = s.strip()
    for name, sset in SYMBOLS.items():
        if letter in sset:
            break
    else:
        if letter == 'k':
            # treat 'k' as an alias for 'K' as per: http://goo.gl/kTQMs
            sset = SYMBOLS['customary']
            letter = letter.upper()
        else:
            raise ValueError("can't interpret %r" % init)
    prefix = {sset[0]: 1}
    for i, s in enumerate(sset[1:]):
        prefix[s] = 1 << (i + 1) * 10
    return int(num * prefix[letter])

## end of http://code.activestate.com/recipes/578019/ }}}


## https://gist.github.com/wy193777/0e2a4932e81afc6aa4c8f7a2984f34e2
def download_from_url(url, dst):
    """
    @param: url to download file
    @param: dst place to put the file
    """

    req = urllib.request.Request(url,  method='HEAD')
    f = urllib.request.urlopen(req)

    file_size = int(f.headers['Content-Length'])
    if os.path.exists(dst):
        first_byte = os.path.getsize(dst)
    else:
        first_byte = 0
    if first_byte >= file_size:
        return file_size
    header = {"Range": "bytes=%s-%s" % (first_byte, file_size)}
    pbar = tqdm(
        total=file_size, initial=first_byte,
        unit='B', unit_scale=True, desc=url.split('/')[-1])
    req = requests.get(url, headers=header, stream=True)
    with(open(dst, 'ab')) as f:
        for chunk in req.iter_content(chunk_size=1024):
            if chunk:
                f.write(chunk)
                pbar.update(1024)
    pbar.close()
    return file_size


import click, os
from assnake.utils import get_config_loc, get_internal_config, pathizer
from assnake.cli.init_config import fill_and_write_config
from assnake.config_internal import config_loc

@click.command(name='start')
@click.option('--assnake_db', '-d', type=click.Path(),
              prompt='Which directory do you like to use for assnake database?',
              help='Desired location of the configiguration file')
@click.option('--fna_db_dir', '-F', type=click.Path(), default='{assnake_db}/fna_db_dir',
              help='This is the place, were all the fasta goes.', required=False)
@click.option('--bwa_index_dir', '-B', type=click.Path(), default='{assnake_db}/bwa_index_dir',
              help='This is the place, were all the bwa index files goes.', required=False)
@click.option('--conda_dir', '-E', type=click.Path(), default='{assnake_db}/conda_dir',
              help='This is the place, were all the conda environments goes.', required=False)
@click.option('--drmaa_log_dir', '-D', type=click.Path(), default='{assnake_db}/drmaa_log_dir',
              help='This is the place, were all the drmaa log files goes.', required=False)
@click.option('--config_location', '-C', type=click.Path(), default='{assnake_db}',
              help='This is the file, were the configure of database will be stored.', required=False)
@click.option('--forced', '-f', is_flag=True,
              help='Force and confirm all in quite mode.')
@click.option('--verbose', '-v', count=True)
# @graph_of_calls('cli_start_init.png')
def init_start(assnake_db, fna_db_dir, bwa_index_dir, conda_dir, drmaa_log_dir, config_location, forced, verbose):
    # TODO Rework assnake install dir
    assnake_db = assnake_db.rstrip('/')

    if config_loc is None or not os.path.isfile(config_loc):
        if not os.path.exists(assnake_db):
            os.mkdir(assnake_db)
        fna_db_dir = fna_db_dir.format(assnake_db=assnake_db)
        bwa_index_dir = bwa_index_dir.format(assnake_db=assnake_db)
        conda_dir = conda_dir.format(assnake_db=assnake_db)
        drmaa_log_dir = drmaa_log_dir.format(assnake_db=assnake_db)
        config_location = config_location.format(assnake_db=assnake_db)
        if verbose > 0:

            def fna_ok():
                return click.confirm(
                    "Proposed folder for fasta files:     " + click.style(fna_db_dir,
                                                                           bold=True,
                                                                           fg='green') + '\n  Is it ok?')

            def bwa_index_ok():
                return click.confirm(
                    "Proposed folder for bwa index files: " + click.style(bwa_index_dir,
                                                                          bold=True,
                                                                          fg='green') + '\n  Is it ok?')

            def conda_ok():
                return click.confirm(
                    "Proposed folder for conda envs:      " + click.style(conda_dir,
                                                                          bold=True,
                                                                          fg='green') + '\n  Is it ok?')

            def log_ok():
                return click.confirm(
                    "Proposed folder for drmaa log files: " + click.style(drmaa_log_dir,
                                                                          bold=True,
                                                                          fg='green') + '\n  Is it ok?')

            def conf_ok():
                return click.confirm(
                    "Proposed path to congig file directory: " + click.style(os.path.join(config_location, 'config.yaml'),
                                                                   bold=True,
                                                                   fg='green') + '\n  Is it ok?')

            while not fna_ok():
                fna_db_dir = click.prompt('Provide absolute path for fasta directory')
            while not bwa_index_ok():
                bwa_index_dir = click.prompt('Provide absolute path for bwa index directory')
            while not conda_ok():
                conda_dir = click.prompt('Provide absolute path for conda environments directory')
            while not log_ok():
                drmaa_log_dir = click.prompt('  Provide absolute path for DRMAA log directory')
            while not conf_ok():
                config_location = click.prompt('  Provide absolute path for configure directory')
            if verbose > 1:
                click.secho(
                    '  Your configuration file will be stored at ' + click.style(os.path.join(config_location, 'config.yaml'), bold=True,
                                                                                 fg='green') + '\n')
        fill_and_write_config(*map(pathizer, [assnake_db, fna_db_dir, bwa_index_dir, conda_dir, drmaa_log_dir, config_location]))
        click.secho('== SUCCESS ==', bg='green', fg='white')
    else:
        if not forced:
            click.echo('We found configuration file at ' + click.style(get_config_loc(), fg='green'))
            if verbose > 0:
                click.secho('Please, keep in mind', err=True)
                click.secho('In (this) case, there is old installation -- only overwriting of ' +
                            click.style('config_internal.ini',
                                        fg='green') + ' in current version is supported.\nDon`t be '
                                                     'upset, cookies are still nice thing and you always can do rm-rf')
            if click.confirm('  Would you like to reset?', abort=True):
                click.echo('As you wish')
        config_internal = get_internal_config()
        dir_of_this_file = os.path.dirname(os.path.abspath(__file__))
        config_internal['GENERAL']['config_loc'] = 'None'
        with open(os.path.join(dir_of_this_file, './../config_internal.ini'), 'w') as configfile:
            config_internal.write(configfile)

@click.command(name = 'current-config')
def current_config():
    """
    Just print your current config location
    """
    click.echo('CURRENT CONFIG LOCATION')
    click.secho(get_config_loc(), bold=True)


import os, yaml
from assnake.utils import read_yaml, get_internal_config


# @graph_of_calls('fill_n_write_config.png')
def fill_and_write_config(assnake_db, fna_db_dir, bwa_index_dir, conda_dir, drmaa_log_dir, config_location):
    # config_internal = get_internal_config()

    dir_of_this_file = os.path.dirname(os.path.abspath(__file__))
    assnake_install_dir = (dir_of_this_file.replace('assnake/commands', 'snake'))

    config_template_loc = os.path.join(dir_of_this_file, '../snake/config_template.yml')

    config_template = read_yaml(config_template_loc)

    config_template['assnake_db'] = assnake_db
    config_template['assnake_install_dir'] = assnake_install_dir
    config_template['fna_db_dir'] = fna_db_dir
    config_template['bwa_index_dir'] = bwa_index_dir
    config_template['conda_dir'] = conda_dir
    config_template['drmaa_log_dir'] = drmaa_log_dir

    os.makedirs(fna_db_dir, exist_ok=True)
    os.makedirs(bwa_index_dir, exist_ok=True)
    os.makedirs(conda_dir, exist_ok=True)
    os.makedirs(drmaa_log_dir, exist_ok=True)
    if not os.path.exists(config_location):
        os.makedirs(config_location, exist_ok=True)

    config_location = os.path.join(config_location, 'config.yaml')
    with open(config_location, 'w+') as file:
        _ = yaml.dump(config_template, file, sort_keys=False)


    with open(os.path.join(dir_of_this_file, './../config_internal.py'), 'w') as configfile:
        configfile.write('config_loc = "' + config_location + '"')
